@@ -1,6 +1,3 @@
-#' @include stack.R
-NULL
-
 ShinySaveState <- R6Class("ShinySaveState",
   public = list(
     input = NULL,
@@ -447,8 +444,8 @@ RestoreInputSet <- R6Class("RestoreInputSet",
   )
 )
 
-
-restoreCtxStack <- Stack$new()
+# This is a fastmap::faststack(); value is assigned in .onLoad().
+restoreCtxStack <- NULL
 
 withRestoreContext <- function(ctx, expr) {
   restoreCtxStack$push(ctx)
@@ -1160,10 +1157,10 @@ setBookmarkExclude <- function(names = character(0), session = getDefaultReactiv
 #'     toupper(input$text)
 #'   })
 #'   onBookmark(function(state) {
-#'     state$values$hash <- digest::digest(input$text, "md5")
+#'     state$values$hash <- rlang::hash(input$text)
 #'   })
 #'   onRestore(function(state) {
-#'     if (identical(digest::digest(input$text, "md5"), state$values$hash)) {
+#'     if (identical(rlang::hash(input$text), state$values$hash)) {
 #'       message("Module's input text matches hash ", state$values$hash)
 #'     } else {
 #'       message("Module's input text does not match hash ", state$values$hash)
@@ -1186,10 +1183,10 @@ setBookmarkExclude <- function(names = character(0), session = getDefaultReactiv
 #' server <- function(input, output, session) {
 #'   callModule(capitalizerServer, "tc")
 #'   onBookmark(function(state) {
-#'     state$values$hash <- digest::digest(input$text, "md5")
+#'     state$values$hash <- rlang::hash(input$text)
 #'   })
 #'   onRestore(function(state) {
-#'     if (identical(digest::digest(input$text, "md5"), state$values$hash)) {
+#'     if (identical(rlang::hash(input$text), state$values$hash)) {
 #'       message("App's input text matches hash ", state$values$hash)
 #'     } else {
 #'       message("App's input text does not match hash ", state$values$hash)
